@@ -1,10 +1,17 @@
 import logging
 from abc import ABC, abstractmethod
+from http import HTTPStatus
 from pathlib import Path
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union, cast
 
 import boto3
 from encode_utils.connection import Connection
+from qc_utils.parsers import (
+    parse_flagstats,
+    parse_hotspot1_spot_score,
+    parse_picard_duplication_metrics,
+    parse_samtools_stats,
+)
 
 from accession.accession_steps import (
     AccessionStep,
@@ -24,13 +31,16 @@ from accession.encode_models import (
     EncodeAnalysis,
     EncodeAttachment,
     EncodeCommonMetadata,
+    EncodeDocument,
+    EncodeDocumentType,
     EncodeExperiment,
     EncodeFile,
+    EncodeGenericObject,
     EncodeQualityMetric,
     EncodeStepRun,
 )
 from accession.file import GSFile
-from accession.helpers import LruCache, flatten, string_to_number
+from accession.helpers import LruCache, flatten, impersonate_file, string_to_number
 from accession.logger_factory import logger_factory
 from accession.metadata import Metadata, metadata_factory
 from accession.preflight import MatchingMd5Record, PreflightHelper
@@ -246,7 +256,8 @@ class Accession(ABC):
         )
         modeled_encode_file = EncodeFile(encode_posted_file)
         if modeled_encode_file.status == "upload failed" or (
-            modeled_encode_file.status == "uploading" and status_code != 409
+            modeled_encode_file.status == "uploading"
+            and status_code != HTTPStatus.CONFLICT
         ):
             self.upload_queue.append((modeled_encode_file, gs_file))
         else:
@@ -385,7 +396,8 @@ class Accession(ABC):
             return_original_status_code=True,
             truncate_long_strings_in_payload_log=True,
         )
-        if status_code == 409:
+        posted, status_code = self.conn.post(payload, return_original_status_code=True)
+        if status_code == HTTPStatus.CONFLICT:
             self.logger.warning(
                 "Analysis step run with aliases %s already exists, will not post it",
                 aliases,
@@ -590,7 +602,7 @@ class Accession(ABC):
         self.raw_qcs.append(modeled_qc)
 
     def get_attachment(
-        self, gs_file: GSFile, mime_type: str, add_ext: str = ""
+        self, gs_file: GSFile, mime_type: str, additional_extension: str = ""
     ) -> Dict[str, str]:
         """
         Files with certain extensions will fail portal validation since it can't guess
@@ -602,38 +614,69 @@ class Accession(ABC):
         filename = gs_file.filename
         contents = self.backend.read_file(filename)
         attachment = EncodeAttachment(contents, filename)
-        obj = attachment.get_portal_object(mime_type, add_ext)
+        obj = attachment.get_portal_object(
+            mime_type, additional_extension=additional_extension
+        )
         return obj
 
-    def patch_experiment_analyses(self) -> None:
+    def post_document(self, document: EncodeDocument) -> EncodeGenericObject:
         """
-        Patch the new analysis (a list of file @ids) to the `analyses` property of the
-        experiment that was being accessioned. If an equivalent analysis exists already,
-        then do not patch anything.
-
-        From the accessioning code standpoint, the analysis is all or nothing: either it
-        is not patched in if it already exists or it is posted after the accessioning of
-        files and qcs completes. As such the code here assumes an analysis is never
-        incomplete.
-
-        encode_utils has `extend_array_values=True` on by default, but we add it here
-        also to be explicit.
+        Returns an instance of `EncodeGenericObject` representing the posted document.
+        If the document already exists, as determined by an alias conflict (409) then
+        the document will not be posted and the existing document on the portal will be
+        returned.
         """
-        current_analysis = EncodeAnalysis.from_files(self.new_files)
-        for analysis in self.experiment.analyses:
-            if current_analysis == analysis:
-                self.logger.info(
-                    "Will not patch analyses for experiment %s, found analysis %s matching the current set of accessioned files %s",
-                    self.experiment.at_id,
-                    analysis,
-                    current_analysis,
-                )
-                return
-        analysis_payload = current_analysis.get_portal_object()
-        payload = self.experiment.make_postable_analyses_from_analysis_payload(
-            analysis_payload
+        postable_document = document.get_portal_object()
+        response, status_code = self.conn.post(
+            postable_document,
+            return_original_status_code=True,
+            truncate_long_strings_in_payload_log=True,
         )
-        self.conn.patch(payload, extend_array_values=True)
+        posted_document = EncodeGenericObject(response)
+        if status_code == HTTPStatus.CONFLICT:
+            self.logger.warning(
+                "Found existing document %s with conflicting aliases, could not post",
+                posted_document.at_id,
+            )
+        return posted_document
+
+    def post_analysis(self) -> EncodeGenericObject:
+        """
+        Tries to POST the new analysis. If an equivalent analysis exists, as determined
+        by 409 conflict, then this will not POST anything. Will post the workflow
+        metadata as an attachment in a document, then insert that document into the
+        `documents` array in the analysis object, and finally post the analyis object.
+        """
+        document_aliases = [
+            f"{self.common_metadata.lab_pi}:cromwell-metadata-{self.analysis.workflow_id}"
+        ]
+        document_attachment = self.analysis.metadata.get_as_attachment()
+        document = EncodeDocument(
+            attachment=document_attachment,
+            common_metadata=self.common_metadata,
+            document_type=EncodeDocumentType.WorkflowMetadata,
+            aliases=document_aliases,
+        )
+        posted_document = self.post_document(document)
+        current_analysis = EncodeAnalysis(
+            files=self.new_files,
+            lab_pi=self.common_metadata.lab_pi,
+            workflow_id=self.analysis.workflow_id,
+            documents=[posted_document],
+        )
+        payload = current_analysis.get_portal_object()
+        response, status_code = self.conn.post(
+            payload,
+            return_original_status_code=True,
+            truncate_long_strings_in_payload_log=True,
+        )
+        modeled_analysis = EncodeGenericObject(response)
+        if status_code == HTTPStatus.CONFLICT:
+            self.logger.warning(
+                "Found existing analysis %s with conflicting aliases, could not post",
+                modeled_analysis.at_id,
+            )
+        return modeled_analysis
 
     def patch_experiment_internal_status(self) -> None:
         """
@@ -643,6 +686,12 @@ class Accession(ABC):
         payload = self.experiment.get_patchable_internal_status()
         self.conn.patch(payload)
 
+    def patch_experiment_analysis_objects(
+        self, analysis_object: EncodeGenericObject
+    ) -> None:
+        payload = self.experiment.get_patchable_analysis_object(analysis_object.at_id)
+        self.conn.patch(payload, extend_array_values=True)
+
     def accession_step(
         self, single_step_params: AccessionStep, dry_run: bool = False
     ) -> Union[List[Optional[MatchingMd5Record]], List[EncodeFile], None]:
@@ -718,6 +767,22 @@ class Accession(ABC):
         First executes a dry run, checking for md5 duplicates. If `dry_run` is `True`
         or if duplicates were detected and `force_accession` is `False` then will
         return, otherwise the experiment will subsequently actually be accessioned.
+
+        The main entrypoint for accessioning. The process looks like this:
+            * For each step in the template, accession the step run, files, and generate
+              the QCs
+            * Post all the QCs generated in the previous step
+            * Post the `Analysis` object pointing to all the files that were accessioned
+              as part of the run
+            * Update the experiment internal_status to indicate that the accessioning
+              has completed
+            * Upload all of the files to S3. We do this as the last step so that all of
+              the metadata objects are posted quickly, which ideally allows them to fit
+              within one Elasticsearch indexing cycle on the portal (1 minute)
+
+        When `dry_run` is `True`, then the only thing we do is iterate through all of
+        the steps and check for potential duplicates that would be posted in the normal
+        mode, without posting, patching, or uploading anything.
         """
         self.logger.info("Currently performing dry run, will not post to server.")
         accumulated_matches = self._get_dry_run_matches()
@@ -744,9 +809,10 @@ class Accession(ABC):
         for step in self.steps.content:
             self.accession_step(step)
         self.post_qcs()
-        self.patch_experiment_analyses()
+        analysis = self.post_analysis()
         for encode_file, gs_file in self.upload_queue:
             self.upload_file(encode_file, gs_file)
+        self.patch_experiment_analysis_objects(analysis)
         self.patch_experiment_internal_status()
 
 
@@ -831,7 +897,7 @@ class AccessionBulkRna(AccessionGenericRna):
         qc_bytes = EncodeAttachment.get_bytes_from_dict(qc)
         modeled_attachment = EncodeAttachment(qc_bytes, gs_file.filename)
         attachment = modeled_attachment.get_portal_object(
-            mime_type="application/json", extension=".json"
+            mime_type="application/json", additional_extension=".json"
         )
         star_qc_metric["attachment"] = attachment
         return self.queue_qc(
@@ -865,7 +931,7 @@ class AccessionBulkRna(AccessionGenericRna):
         qc_bytes = EncodeAttachment.get_bytes_from_dict(qc)
         modeled_attachment = EncodeAttachment(qc_bytes, gs_file.filename)
         attachment = modeled_attachment.get_portal_object(
-            mime_type="application/json", extension=".json"
+            mime_type="application/json", additional_extension=".json"
         )
         output_qc["attachment"] = attachment
         return self.queue_qc(
@@ -919,7 +985,7 @@ class AccessionBulkRna(AccessionGenericRna):
         qc_bytes = EncodeAttachment.get_bytes_from_dict(qc)
         modeled_attachment = EncodeAttachment(qc_bytes, gs_file.filename)
         attachment = modeled_attachment.get_portal_object(
-            mime_type="application/json", extension=".json"
+            mime_type="application/json", additional_extension=".json"
         )
         output_qc["attachment"] = attachment
         return self.queue_qc(
@@ -974,6 +1040,268 @@ class AccessionBulkRna(AccessionGenericRna):
         return mad_qc
 
 
+class AccessionDnase(Accession):
+    QC_MAP = {
+        "unfiltered_flagstats": "make_unfiltered_flagstats_qc",
+        "unfiltered_trimstats": "make_unfiltered_trimstats_qc",
+        "nuclear_flagstats": "make_nuclear_flagstats_qc",
+        "nuclear_duplication_metric": "make_nuclear_duplication_qc",
+        "nuclear_hotspot1_metric": "make_nuclear_hotspot1_qc",
+        "nuclear_samtools_stats": "make_nuclear_samtools_stats_qc",
+        "nuclear_alignment_quality_metric": "make_nuclear_alignment_qc",
+        "footprints_quality_metric": "make_footprints_qc",
+        "tenth_of_one_percent_peaks_qc": "make_tenth_of_one_percent_peaks_qc",
+        "five_percent_allcalls_qc": "make_five_percent_allcalls_qc",
+        "five_percent_narrowpeaks_qc": "make_five_percent_narrowpeaks_qc",
+    }  # type: ignore
+
+    @property
+    def assembly(self) -> str:
+        filekey = "references.nuclear_chroms_gz"
+        return self.find_portal_property_from_filekey(filekey, EncodeFile.ASSEMBLY)
+
+    def parse_dict_from_bytes(self, qc_bytes: bytes, parser) -> dict:
+        with impersonate_file(qc_bytes) as fake_file:
+            result = parser(fake_file)
+        return result
+
+    def make_flagstats_qc(
+        self, encode_file: EncodeFile, gs_file: GSFile, filekey: str
+    ) -> None:
+        if encode_file.has_qc("SamtolsFlagstatsQualityMetric"):
+            return
+        qc_file = self.analysis.get_files(filekey=filekey)[0]  # this is GSFile
+        qc_bytes = self.backend.read_file(qc_file.filename)
+        with impersonate_file(qc_bytes) as flagstats:
+            qc_output_dict = parse_flagstats(flagstats)
+        qc_output_dict["mapped_pct"] = str(qc_output_dict["mapped_pct"])
+        qc_output_dict["paired_properly_pct"] = str(
+            qc_output_dict["paired_properly_pct"]
+        )
+        qc_output_dict["singletons_pct"] = str(qc_output_dict["singletons_pct"])
+        attachment = self.get_attachment(qc_file, "text/plain")
+        qc_output_dict["attachment"] = attachment
+        return self.queue_qc(
+            qc_output_dict, encode_file, "samtools-flagstats-quality-metric"
+        )
+
+    def make_unfiltered_flagstats_qc(
+        self, encode_file: EncodeFile, gs_file: GSFile
+    ) -> None:
+        self.make_flagstats_qc(
+            encode_file=encode_file,
+            gs_file=gs_file,
+            filekey="analysis.qc.unfiltered_bam_qc.flagstats",
+        )
+
+    def make_unfiltered_trimstats_qc(
+        self, encode_file: EncodeFile, gs_gile: GSFile
+    ) -> None:
+        qc_output_dict = {}
+        if encode_file.has_qc("TrimmingQualityMetric"):
+            return
+        qc_file = self.analysis.get_files(
+            filekey="analysis.qc.unfiltered_bam_qc.trimstats"
+        )[0]
+        attachment = self.get_attachment(qc_file, "text/plain")
+        qc_output_dict["attachment"] = attachment
+        return self.queue_qc(qc_output_dict, encode_file, "trimming-quality-metric")
+
+    def make_nuclear_flagstats_qc(
+        self, encode_file: EncodeFile, gs_file: GSFile
+    ) -> None:
+        self.make_flagstats_qc(
+            encode_file=encode_file,
+            gs_file=gs_file,
+            filekey="analysis.qc.nuclear_bam_qc.flagstats",
+        )
+
+    def make_nuclear_duplication_qc(
+        self, encode_file: EncodeFile, gs_file: GSFile
+    ) -> None:
+        if encode_file.has_qc("DuplicatesQualityMetric"):
+            return
+        qc_file = self.analysis.get_files(
+            filekey="analysis.qc.nuclear_bam_qc.duplication_metrics"
+        )[0]
+        qc_bytes = self.backend.read_file(qc_file.filename)
+        qc_output_dict = self.parse_dict_from_bytes(
+            qc_bytes, parse_picard_duplication_metrics
+        )
+        attachment = self.get_attachment(
+            qc_file, "text/plain", additional_extension=".txt"
+        )
+        qc_output_dict["attachment"] = attachment
+        return self.queue_qc(qc_output_dict, encode_file, "duplicates-quality-metric")
+
+    def make_nuclear_hotspot1_qc(
+        self, encode_file: EncodeFile, gs_file: GSFile
+    ) -> None:
+        if encode_file.has_qc("HotspotQualityMetric"):
+            return
+        qc_file = self.analysis.get_files(
+            filekey="analysis.qc.nuclear_bam_qc.hotspot1"
+        )[0]
+        qc_bytes = self.backend.read_file(qc_file.filename)
+        qc_output_dict = self.parse_dict_from_bytes(qc_bytes, parse_hotspot1_spot_score)
+        attachment = self.get_attachment(
+            qc_file, "text/plain", additional_extension=".txt"
+        )
+        qc_output_dict["attachment"] = attachment
+        return self.queue_qc(qc_output_dict, encode_file, "hotspot-quality-metric")
+
+    def make_nuclear_samtools_stats_qc(
+        self, encode_file: EncodeFile, gs_file: GSFile
+    ) -> None:
+        if encode_file.has_qc("SamtoolStatsQualityMetric"):
+            return
+        qc_file = self.analysis.get_files(filekey="analysis.qc.nuclear_bam_qc.stats")[0]
+        qc_bytes = self.backend.read_file(qc_file.filename)
+        qc_output_dict = self.parse_dict_from_bytes(qc_bytes, parse_samtools_stats)
+        attachment = self.get_attachment(qc_file, "text/plain")
+        qc_output_dict["attachment"] = attachment
+        non_encode_keys = [
+            "total first fragment length",
+            "total last fragment length",
+            "average first fragment length",
+            "average last fragment length",
+            "maximum first fragment length",
+            "maximum last fragment length",
+            "percentage of properly paired reads (%)",
+        ]
+        for key in non_encode_keys:
+            del qc_output_dict[key]
+        return self.queue_qc(
+            qc_output_dict, encode_file, "samtools-stats-quality-metric"
+        )
+
+    def make_nuclear_alignment_qc(
+        self, encode_file: EncodeFile, gs_file: GSFile
+    ) -> None:
+        if encode_file.has_qc("DnaseAlignmentQualityMetric"):
+            return
+        dnase_alignment_qc_output = {}
+        insert_size_info_file = self.analysis.get_files(
+            filekey="analysis.qc.nuclear_bam_qc.insert_size_info"
+        )[0]
+        insert_size_info_attachment = self.get_attachment(
+            insert_size_info_file, "text/plain", additional_extension=".txt"
+        )
+        dnase_alignment_qc_output["attachment"] = insert_size_info_attachment
+        insert_size_metric_file = self.analysis.get_files(
+            filekey="analysis.qc.nuclear_bam_qc.insert_size_metrics"
+        )[0]
+        insert_size_metric_attachment = self.get_attachment(
+            insert_size_metric_file, "text/plain", additional_extension=".txt"
+        )
+        dnase_alignment_qc_output["insert_size_metric"] = insert_size_metric_attachment
+        nuclear_preseq_file = self.analysis.get_files(
+            filekey="analysis.qc.nuclear_bam_qc.preseq"
+        )[0]
+        nuclear_preseq_attachment = self.get_attachment(
+            nuclear_preseq_file, "text/plain"
+        )
+        dnase_alignment_qc_output["nuclear_preseq"] = nuclear_preseq_attachment
+        nuclear_preseq_targets_file = self.analysis.get_files(
+            filekey="analysis.qc.nuclear_bam_qc.preseq_targets"
+        )[0]
+        nuclear_preseq_targets_attachment = self.get_attachment(
+            nuclear_preseq_targets_file, "text/plain"
+        )
+        dnase_alignment_qc_output[
+            "nuclear_preseq_targets"
+        ] = nuclear_preseq_targets_attachment
+        insert_size_histogram_file = self.analysis.get_files(
+            filekey="analysis.qc.nuclear_bam_qc.insert_size_histogram_pdf"
+        )[0]
+        insert_size_histogram_attachment = self.get_attachment(
+            insert_size_histogram_file, "application/pdf"
+        )
+        dnase_alignment_qc_output[
+            "insert_size_histogram"
+        ] = insert_size_histogram_attachment
+        return self.queue_qc(
+            dnase_alignment_qc_output, encode_file, "dnase-alignment-quality-metric"
+        )
+
+    def make_footprints_qc(self, encode_file: EncodeFile, gs_file: GSFile) -> None:
+
+        if encode_file.has_qc("DnaseFootprintingQualityMetric"):
+            return
+        footprint_count = int(
+            gs_file.task.outputs["analysis"]["qc"]["footprints_qc"][
+                "one_percent_footprints_count"
+            ]
+        )
+        dispersion_model_file = self.analysis.get_files(
+            filekey="analysis.qc.footprints_qc.dispersion_model"
+        )[0]
+        dispersion_model_attachment = self.get_attachment(
+            dispersion_model_file, "application/json"
+        )
+        footprints_qc_output = {}  # type: Dict[str, Union[int, Dict[str,str]]]
+        footprints_qc_output["footprint_count"] = footprint_count
+        footprints_qc_output["dispersion_model"] = dispersion_model_attachment
+        return self.queue_qc(
+            footprints_qc_output, encode_file, "dnase-footprinting-quality-metric"
+        )
+
+    def make_tenth_of_one_percent_peaks_qc(
+        self, encode_file: EncodeFile, gs_file: GSFile
+    ) -> None:
+        if encode_file.has_qc("HotspotsQualityMetric"):
+            return
+        tenth_of_percent_narrowpeaks_count = int(
+            gs_file.task.outputs["analysis"]["qc"]["peaks_qc"][
+                "tenth_of_one_percent_narrowpeaks_count"
+            ]
+        )
+        qc_output = {}
+        qc_output[
+            "tenth_of_one_percent_narrowpeaks_count"
+        ] = tenth_of_percent_narrowpeaks_count
+        return self.queue_qc(qc_output, encode_file, "hotspot-quality-metric")
+
+    def make_five_percent_allcalls_qc(
+        self, encode_file: EncodeFile, gs_file: GSFile
+    ) -> None:
+        if encode_file.has_qc("HotspotQualityMetric"):
+            return
+        five_percent_allcalls_count = int(
+            gs_file.task.outputs["analysis"]["qc"]["peaks_qc"][
+                "five_percent_allcalls_count"
+            ]
+        )
+        qc_output = {}
+        qc_output["five_percent_allcalls_count"] = five_percent_allcalls_count
+        return self.queue_qc(qc_output, encode_file, "hotspot-quality-metric")
+
+    def make_five_percent_narrowpeaks_qc(
+        self, encode_file: EncodeFile, gs_file: GSFile
+    ) -> None:
+        if encode_file.has_qc("HotspotQualityMetric"):
+            return
+        five_percent_narrowpeaks_count = int(
+            gs_file.task.outputs["analysis"]["qc"]["peaks_qc"][
+                "five_percent_narrowpeaks_count"
+            ]
+        )
+        five_percent_hotspots_count = int(
+            gs_file.task.outputs["analysis"]["qc"]["peaks_qc"][
+                "five_percent_hotspots_count"
+            ]
+        )
+        hotspot2_file = self.analysis.get_files(
+            filekey="analysis.qc.peaks_qc.hotspot2"
+        )[0]
+        hotspot2_score = float(self.backend.read_file(hotspot2_file.filename).decode())
+        qc_output = {}  # type: Dict[str, Union[int, float]]
+        qc_output["five_percent_narrowpeaks_count"] = five_percent_narrowpeaks_count
+        qc_output["five_percent_hotspots_count"] = five_percent_hotspots_count
+        qc_output["spot2_score"] = hotspot2_score
+        return self.queue_qc(qc_output, encode_file, "hotspot-quality-metric")
+
+
 class AccessionLongReadRna(AccessionGenericRna):
     QC_MAP = {
         "long_read_rna_mapping": "make_long_read_rna_mapping_qc",
@@ -1210,7 +1538,7 @@ class AccessionAtacChip(Accession):
             )
         ]
         pipeline_rep = None
-        for k, v in self.analysis.metadata["inputs"].items():
+        for k, v in self.analysis.metadata.content["inputs"].items():
             if "fastqs" in k and "ctl" not in k:
                 if sorted(v) == sorted(parent_fastqs):
                     pipeline_rep = k.split("_")[1]
@@ -1525,7 +1853,7 @@ class AccessionChip(AccessionAtacChip):
             output_qc.update(
                 {
                     "idr_parameters": self.get_attachment(
-                        idr_log, "text/plain", add_ext=".txt"
+                        idr_log, "text/plain", additional_extension=".txt"
                     )
                 }
             )
@@ -1738,7 +2066,7 @@ class AccessionAtac(AccessionAtacChip):
             output_qc.update(
                 {
                     "idr_parameters": self.get_attachment(
-                        idr_log, "text/plain", add_ext=".txt"
+                        idr_log, "text/plain", additional_extension=".txt"
                     )
                 }
             )
@@ -1806,6 +2134,7 @@ def accession_factory(
         "mint_chip": AccessionChip,
         "control_chip": AccessionChip,
         "atac": AccessionAtac,
+        "dnase": AccessionDnase,
     }
     selected_accession: Optional[Type[Accession]] = None
     try:
@@ -1829,7 +2158,10 @@ def accession_factory(
     accession_steps = AccessionSteps(steps_json_path)
     backend = kwargs.pop("backend", None)
     analysis = Analysis(
-        metadata, raw_fastqs_keys=accession_steps.raw_fastqs_keys, backend=backend
+        metadata,
+        raw_fastqs_keys=accession_steps.raw_fastqs_keys,
+        raw_fastqs_can_have_task=accession_steps.raw_fastqs_can_have_task,
+        backend=backend,
     )
     connection = Connection(server, no_log_file=True)
     common_metadata = EncodeCommonMetadata(lab, award)
