@@ -1,6 +1,7 @@
 import json
 import os
 import shutil
+from base64 import b64decode
 from pathlib import Path
 from time import sleep
 from typing import Callable, Dict, Iterator, Tuple
@@ -12,19 +13,26 @@ import encode_utils as eu
 import pytest
 import requests
 from encode_utils.connection import Connection
-from google.cloud import storage
 from pytest_mock.plugin import MockFixture
 
 import docker
+from accession import backends
 from accession.accession import (
     Accession,
     AccessionChip,
     AccessionMicroRna,
     accession_factory,
 )
-from accession.analysis import Analysis, MetaData
-from accession.backends import GCBackend
-from accession.encode_models import EncodeCommonMetadata, EncodeExperiment, EncodeFile
+from accession.analysis import Analysis
+from accession.encode_models import (
+    EncodeAttachment,
+    EncodeCommonMetadata,
+    EncodeDocument,
+    EncodeDocumentType,
+    EncodeExperiment,
+    EncodeFile,
+)
+from accession.metadata import FileMetadata
 
 
 @pytest.fixture(scope="session")
@@ -96,7 +104,6 @@ def local_encoded_server(
             continue
         container.kill()
         raise RuntimeError("Elasticsearch took too long to index")
-    sleep(10)
     yield server_address
     container.kill()
 
@@ -147,7 +154,9 @@ def metadata_json_path() -> Path:
 def normal_analysis(
     mock_gc_backend: MockFixture, metadata_json_path: Path
 ) -> Analysis:  # noqa: F811
-    normal_analysis = Analysis(MetaData(metadata_json_path), backend=mock_gc_backend)
+    normal_analysis = Analysis(
+        FileMetadata(metadata_json_path), backend=mock_gc_backend
+    )
     return normal_analysis
 
 
@@ -169,6 +178,28 @@ def encode_file_no_qc():
     return EncodeFile({"@id": "/files/foo/", "quality_metrics": []})
 
 
+@pytest.fixture(scope="module")
+def encode_attachment():
+    return EncodeAttachment(
+        filename="/my/dir/haz/my_text_file", contents=b"foo bar baz"
+    )
+
+
+@pytest.fixture(scope="module")
+def encode_common_metadata():
+    return EncodeCommonMetadata("/labs/lab/", "award")
+
+
+@pytest.fixture
+def encode_document(encode_attachment, encode_common_metadata):
+    return EncodeDocument(
+        encode_attachment,
+        encode_common_metadata,
+        EncodeDocumentType.WorkflowMetadata,
+        aliases=["encode:foo"],
+    )
+
+
 @attr.s(auto_attribs=True)
 class MockFile:
     filename: str
@@ -181,11 +212,10 @@ def mock_file() -> MockFile:
     return MockFile("gs://foo/bar", 123, "abc")
 
 
-class MockGCBackend(GCBackend):
+class MockGCBackend(backends.GCBackend):
     def __init__(self, bucket: str):
         self.client = MockGCClient()
         self.bucket = self.client.get_bucket(bucket)
-        self.local_mapping = {}
 
 
 class MockBucket:
@@ -212,9 +242,9 @@ class MockBlob:
     size: int = 3
 
     @property
-    def md5_hash(self) -> str:
-        # c8dd0119389ce1e83eca7ecadc15651b in hex
-        return "yN0BGTic4eg+yn7K3BVlGw=="
+    def md5sum(self) -> str:
+        # yN0BGTic4eg+yn7K3BVlGw== in base 64
+        return "c8dd0119389ce1e83eca7ecadc15651b"
 
     @property
     def id(self) -> str:
@@ -231,10 +261,6 @@ class MockBlob:
     def download_as_string(self) -> bytes:
         return b'{"foobar": "bazqux"}'
 
-    def download_to_filename(self, file: str) -> None:
-        with open(file, "wb") as f:
-            f.write(b'{"foobar": "bazqux"}')
-
 
 def load_md5_map() -> Dict[str, str]:
     md5_file = Path(__file__).resolve().parent / "data" / "gcloud_md5s.json"
@@ -252,8 +278,9 @@ class LocalMockBlob(MockBlob):
     md5_lookup = load_md5_map()
 
     @property
-    def md5_hash(self) -> str:
-        return self.md5_lookup[f"gs://{self.bucket.name}/{self.path}"]
+    def md5sum(self) -> str:
+        b64_md5 = self.md5_lookup[f"gs://{self.bucket.name}/{self.path}"]
+        return b64decode(b64_md5).hex()
 
     def download_as_string(self) -> bytes:
         """
@@ -277,7 +304,7 @@ def mock_gc_backend(mocker) -> MockGCBackend:
     mocker fixture can only be used with function-scoped pytest fixtures for now:
     https://github.com/pytest-dev/pytest-mock/issues/136
     """
-    mocker.patch.object(storage.blob, "Blob", MockBlob)
+    mocker.patch.object(backends, "GcsBlob", MockBlob)
     mock_gc_backend = MockGCBackend("accession-test-bucket")
     return mock_gc_backend
 
@@ -288,7 +315,7 @@ def mock_accession_gc_backend(mocker) -> MockGCBackend:
     Similar to mock_gc_backend, except it provides real md5sums and can access real test
     data files.
     """
-    mocker.patch.object(storage.blob, "Blob", LocalMockBlob)
+    mocker.patch.object(backends, "GcsBlob", LocalMockBlob)
     mock_gc_backend = MockGCBackend("encode-processing")
     return mock_gc_backend
 
@@ -319,10 +346,11 @@ def accessioner_factory(
             lab,
             award,
             backend=mock_accession_gc_backend,
+            no_log_file=True,
         )
 
         mocker.patch.object(
-            accessioner.conn, "upload_file", return_value=None, autospec=True
+            accessioner, "upload_file", return_value=None, autospec=True
         )
         with open(
             current_dir.parent / "data" / "validation" / f"{assay_name}" / "files.json"
@@ -330,18 +358,18 @@ def accessioner_factory(
             expected_files = json.load(f)
         yield (accessioner, expected_files)
         shutil.rmtree(current_dir.parents[1] / "EU_Logs")
-        os.remove(current_dir.parents[1] / "accession.log")
 
     return _accessioner_factory
 
 
-class MockMetaData:
-    content = {"workflowRoot": "gs://foo/bar", "calls": {}}
+class MockMetadata:
+    content = {"id": "foo", "workflowRoot": "gs://foo/bar", "calls": {}}
+    workflow_id = "123"
 
 
 @pytest.fixture
 def mock_metadata():
-    return MockMetaData()
+    return MockMetadata()
 
 
 class MockAccessionSteps:
@@ -358,7 +386,7 @@ def mock_accession_steps():
 def mock_accession(
     mocker: MockFixture,
     mock_accession_gc_backend: MockGCBackend,
-    mock_metadata: MockMetaData,
+    mock_metadata: MockMetadata,
     mock_accession_steps: MockAccessionSteps,
     server_name: str,
     common_metadata: EncodeCommonMetadata,
@@ -387,7 +415,6 @@ def mock_accession(
                         {"biological_replicate_number": 1},
                         {"biological_replicate_number": 2},
                     ],
-                    "analyses": [{"files": ["/files/1/"]}],
                 }
             )
         ),
@@ -430,7 +457,7 @@ def mock_accession_not_patched(
 def mock_accession_chip(
     mocker: MockFixture,
     mock_accession_gc_backend: MockGCBackend,
-    mock_metadata: MockMetaData,
+    mock_metadata: MockMetadata,
     mock_accession_steps: MockAccessionSteps,
     server_name: str,
     common_metadata: EncodeCommonMetadata,
@@ -477,7 +504,7 @@ def mock_accession_chip(
 def mock_accession_unreplicated(
     mocker: MockFixture,
     mock_accession_gc_backend: MockGCBackend,
-    mock_metadata: MockMetaData,
+    mock_metadata: MockMetadata,
     lab: str,
     award: str,
 ) -> Accession:
